@@ -531,6 +531,15 @@ test_empty_asx (void)
 	g_free (uri);
 }
 
+static void
+test_empty_pls (void)
+{
+	char *uri;
+	uri = get_relative_uri (TEST_SRCDIR "emptyplaylist.pls");
+	g_assert (simple_parser_test (uri) == TOTEM_PL_PARSER_RESULT_UNHANDLED);
+	g_free (uri);
+}
+
 static void
 test_parsing_rtsp_text_multi (void)
 {
@@ -550,6 +559,15 @@ test_parsing_rtsp_text (void)
 	g_free (uri);
 }
 
+static void
+test_parsing_content_type (void)
+{
+	char *uri;
+	uri = get_relative_uri (TEST_SRCDIR "HackerMedley");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_CONTENT_TYPE), ==, "audio/mpeg");
+	g_free (uri);
+}
+
 static void
 test_parsing_hadess (void)
 {
@@ -578,6 +596,17 @@ test_parsing_broken_asx (void)
 	g_assert (result != TOTEM_PL_PARSER_RESULT_ERROR);
 }
 
+static void
+test_xml_is_text_plain (void)
+{
+	TotemPlParserResult result;
+
+	g_test_bug ("655378");
+	result = simple_parser_test ("http://leoville.tv/podcasts/floss.xml");
+	g_message ("result %d", result);
+	g_assert (result == TOTEM_PL_PARSER_RESULT_SUCCESS);
+}
+
 static void
 test_parsing_out_of_order_asx (void)
 {
@@ -941,6 +970,7 @@ main (int argc, char *argv[])
 		g_test_add_func ("/parser/resolution", test_resolution);
 		g_test_add_func ("/parser/parsability", test_parsability);
 		g_test_add_func ("/parser/videosite", test_videosite);
+		g_test_add_func ("/parser/xml_is_text_plain", test_xml_is_text_plain);
 		g_test_add_func ("/parser/parsing/hadess", test_parsing_hadess);
 		g_test_add_func ("/parser/parsing/nonexistent_files", test_parsing_nonexistent_files);
 		g_test_add_func ("/parser/parsing/broken_asx", test_parsing_broken_asx);
@@ -952,6 +982,7 @@ main (int argc, char *argv[])
 		g_test_add_func ("/parser/parsing/xml_comment_whitespace", test_parsing_xml_comment_whitespace);
 		g_test_add_func ("/parser/parsing/multi_line_rtsptext", test_parsing_rtsp_text_multi);
 		g_test_add_func ("/parser/parsing/single_line_rtsptext", test_parsing_rtsp_text);
+		g_test_add_func ("/parser/parsing/podcast_content_type", test_parsing_content_type);
 		g_test_add_func ("/parser/parsing/live_streaming", test_parsing_live_streaming);
 		g_test_add_func ("/parser/parsing/xml_mixed_cdata", test_parsing_xml_mixed_cdata);
 		g_test_add_func ("/parser/parsing/not_asx_playlist", test_parsing_not_asx_playlist);
@@ -965,6 +996,7 @@ main (int argc, char *argv[])
 		g_test_add_func ("/parser/parsing/smi_starttime", test_smi_starttime);
 		g_test_add_func ("/parser/parsing/m3u_leading_tabs", test_m3u_leading_tabs);
 		g_test_add_func ("/parser/parsing/empty-asx.asx", test_empty_asx);
+		g_test_add_func ("/parser/parsing/emptyplaylist.pls", test_empty_pls);
 		g_test_add_func ("/parser/parsing/dir_recurse", test_directory_recurse);
 		g_test_add_func ("/parser/parsing/async_signal_order", test_async_parsing_signal_order);
 		g_test_add_func ("/parser/parsing/wma_asf", test_parsing_wma_asf);
