@@ -27,7 +27,7 @@
 
 #ifndef TOTEM_PL_PARSER_MINI
 #include <glib/gi18n-lib.h>
-#include <gtk/gtk.h>
+
 #include <gio/gio.h>
 
 #include "totem-pl-parser.h"
@@ -78,13 +78,15 @@ totem_pl_parser_uri_to_dos (const char *uri, GFile *output)
 }
 
 gboolean
-totem_pl_parser_write_m3u (TotemPlParser *parser, GtkTreeModel *model,
-			   TotemPlParserIterFunc func, GFile *output,
-			   gboolean dos_compatible, gpointer user_data, GError **error)
+totem_pl_parser_save_m3u (TotemPlParser    *parser,
+                          TotemPlPlaylist  *playlist,
+                          GFile            *output,
+                          gboolean          dos_compatible,
+                          GError          **error)
 {
+        TotemPlPlaylistIter iter;
 	GFileOutputStream *stream;
-	int num_entries_total, i;
-	gboolean success;
+	gboolean valid, success;
 	char *buf;
 	char *cr;
 
@@ -93,9 +95,6 @@ totem_pl_parser_write_m3u (TotemPlParser *parser, GtkTreeModel *model,
 		return FALSE;
 
 	cr = dos_compatible ? "\r\n" : "\n";
-	num_entries_total = gtk_tree_model_iter_n_children (model, NULL);
-	if (num_entries_total == 0)
-		return TRUE;
 
 	buf = g_strdup_printf ("#EXTM3U%s", cr);
 	success = totem_pl_parser_write_string (G_OUTPUT_STREAM (stream), buf, error);
@@ -103,18 +102,26 @@ totem_pl_parser_write_m3u (TotemPlParser *parser, GtkTreeModel *model,
 	if (success == FALSE)
 		return FALSE;
 
-	for (i = 1; i <= num_entries_total; i++) {
-		GtkTreeIter iter;
+        valid = totem_pl_playlist_iter_first (playlist, &iter);
+
+        while (valid) {
 		char *uri, *title, *path2;
-		gboolean custom_title;
 		GFile *file;
 
-		if (gtk_tree_model_iter_nth_child (model, &iter, NULL, i - 1) == FALSE)
-			continue;
+                totem_pl_playlist_get (playlist, &iter,
+                                       TOTEM_PL_PARSER_FIELD_URI, &uri,
+                                       TOTEM_PL_PARSER_FIELD_TITLE, &title,
+                                       NULL);
 
-		func (model, &iter, &uri, &title, &custom_title, user_data);
+                valid = totem_pl_playlist_iter_next (playlist, &iter);
+
+                if (!uri) {
+                        g_free (title);
+                        continue;
+                }
+
+                file = g_file_new_for_uri (uri);
 
-		file = g_file_new_for_uri (uri);
 		if (totem_pl_parser_scheme_is_ignored (parser, file) != FALSE) {
 			g_object_unref (file);
 			g_free (uri);
@@ -123,7 +130,7 @@ totem_pl_parser_write_m3u (TotemPlParser *parser, GtkTreeModel *model,
 		}
 		g_object_unref (file);
 
-		if (custom_title != FALSE) {
+		if (title) {
 			buf = g_strdup_printf (EXTINF",%s%s", title, cr);
 			success = totem_pl_parser_write_string (G_OUTPUT_STREAM (stream), buf, error);
 			g_free (buf);
@@ -244,24 +251,17 @@ totem_pl_parser_parse_ram_uri (TotemPlParser *parser, const char *uri)
 }
 
 TotemPlParserResult
-totem_pl_parser_add_ram (TotemPlParser *parser, GFile *file, gpointer data)
+totem_pl_parser_add_ram (TotemPlParser *parser, GFile *file, TotemPlParseData *parse_data, gpointer data)
 {
 	gboolean retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
 	char *contents, **lines;
 	gsize size;
 	guint i;
-	const char *split_char;
 
 	if (g_file_load_contents (file, NULL, &contents, &size, NULL, NULL) == FALSE)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
-	/* figure out whether we're a unix or dos RAM file */
-	if (strstr(contents,"\x0d") == NULL)
-		split_char = "\n";
-	else
-		split_char = "\x0d\n";
-
-	lines = g_strsplit (contents, split_char, 0);
+	lines = g_strsplit_set (contents, "\r\n", 0);
 	g_free (contents);
 
 	for (i = 0; lines[i] != NULL; i++) {
@@ -278,7 +278,7 @@ totem_pl_parser_add_ram (TotemPlParser *parser, GFile *file, gpointer data)
 
 			line_file = g_file_new_for_uri (lines[i]);
 			/* .ram files can contain .smil entries */
-			if (totem_pl_parser_parse_internal (parser, line_file, NULL) != TOTEM_PL_PARSER_RESULT_SUCCESS)
+			if (totem_pl_parser_parse_internal (parser, line_file, NULL, parse_data) != TOTEM_PL_PARSER_RESULT_SUCCESS)
 				totem_pl_parser_parse_ram_uri (parser, lines[i]);
 			g_object_unref (line_file);
 		} else if (strcmp (lines[i], "--stop--") == 0) {
@@ -322,13 +322,6 @@ totem_pl_parser_get_extinfo_title (const char *extinfo)
 	if (res[0] == '\0')
 		return NULL;
 
-	/* Handle ':' as a field separator */
-	sep = strstr (res, ":");
-	if (sep != NULL && sep[1] != '\0') {
-		sep++;
-		return sep;
-	}
-
 	/* Handle ',' as a field separator */
 	sep = strstr (res, ",");
 	if (sep == NULL || sep[1] == '\0') {
@@ -345,14 +338,15 @@ TotemPlParserResult
 totem_pl_parser_add_m3u (TotemPlParser *parser,
 			 GFile *file,
 			 GFile *base_file,
+			 TotemPlParseData *parse_data,
 			 gpointer data)
 {
 	TotemPlParserResult retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
 	char *contents, **lines;
 	gsize size;
 	guint i, num_lines;
-	gboolean dos;
-	const char *split_char, *extinfo;
+	gboolean dos_mode = FALSE;
+	const char *extinfo;
 
 	if (g_file_load_contents (file, NULL, &contents, &size, NULL, NULL) == FALSE)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
@@ -361,7 +355,7 @@ totem_pl_parser_add_m3u (TotemPlParser *parser,
 	if (g_str_has_prefix (contents, "[playlist]") != FALSE
 			|| g_str_has_prefix (contents, "[Playlist]") != FALSE
 			|| g_str_has_prefix (contents, "[PLAYLIST]") != FALSE) {
-		retval = totem_pl_parser_add_pls_with_contents (parser, file, base_file, contents);
+		retval = totem_pl_parser_add_pls_with_contents (parser, file, base_file, contents, parse_data);
 		g_free (contents);
 		return retval;
 	}
@@ -381,64 +375,63 @@ totem_pl_parser_add_m3u (TotemPlParser *parser,
 	extinfo = NULL;
 
 	/* figure out whether we're a unix m3u or dos m3u */
-	if (strstr(contents,"\x0d") == NULL) {
-		split_char = "\n";
-		dos = FALSE;
-	} else {
-		split_char = "\x0d\n";
-		dos = TRUE;
+	if (strstr(contents, "\x0d")) {
+		dos_mode = TRUE;
 	}
 
-	lines = g_strsplit_set (contents, split_char, 0);
+	lines = g_strsplit_set (contents, "\r\n", 0);
 	g_free (contents);
 	num_lines = g_strv_length (lines);
 	/* We don't count the terminating NULL */
 	num_lines--;
 
 	for (i = 0; lines[i] != NULL; i++) {
-		if (lines[i][0] == '\0')
+		const char *line;
+
+		line = lines[i];
+
+		if (line[0] == '\0')
 			continue;
 
 		retval = TOTEM_PL_PARSER_RESULT_SUCCESS;
 
+		/* Ignore leading spaces */
+		for (; g_ascii_isspace (line[0]); line++)
+			;
+
 		/* Ignore comments, but mark it if we have extra info */
-		if (lines[i][0] == '#') {
-			if (extinfo == NULL && g_str_has_prefix (lines[i], EXTINF) != FALSE)
-				extinfo = lines[i];
+		if (line[0] == '#') {
+			if (extinfo == NULL && g_str_has_prefix (line, EXTINF) != FALSE)
+				extinfo = line;
 			continue;
 		}
 
-		/* If we're on the last line, and we're a DOS M3U,
-		 * remove the terminating \n if it's there */
-		if (dos != FALSE && i == num_lines)
-			g_strchomp (lines[i]);
-
 		/* Either it's a URI, or it has a proper path ... */
-		if (strstr(lines[i], "://") != NULL
-				|| lines[i][0] == G_DIR_SEPARATOR) {
+		if (strstr(line, "://") != NULL
+				|| line[0] == G_DIR_SEPARATOR) {
 			GFile *uri;
 
-			uri = g_file_new_for_commandline_arg (lines[i]);
-			if (totem_pl_parser_parse_internal (parser, uri, NULL) != TOTEM_PL_PARSER_RESULT_SUCCESS) {
-				totem_pl_parser_add_one_uri (parser, lines[i],
+			uri = g_file_new_for_commandline_arg (line);
+			if (totem_pl_parser_parse_internal (parser, uri, NULL, parse_data) != TOTEM_PL_PARSER_RESULT_SUCCESS) {
+				totem_pl_parser_add_one_uri (parser, line,
 						totem_pl_parser_get_extinfo_title (extinfo));
 			}
 			g_object_unref (uri);
 			extinfo = NULL;
-		} else if (g_ascii_isalpha (lines[i][0]) != FALSE
-			   && g_str_has_prefix (lines[i] + 1, ":\\")) {
+		} else if (g_ascii_isalpha (line[0]) != FALSE
+			   && g_str_has_prefix (line + 1, ":\\")) {
 			/* Path relative to a drive on Windows, we need to use
 			 * the base that was passed to us */
 			GFile *uri;
 
 			lines[i] = g_strdelimit (lines[i], "\\", '/');
 			/* + 2, skip drive letter */
-			uri = g_file_get_child (base_file, lines[i] + 2);
+			uri = g_file_get_child (base_file, line + 2);
 			totem_pl_parser_add_one_file (parser, uri,
 						     totem_pl_parser_get_extinfo_title (extinfo));
 			g_object_unref (uri);
 			extinfo = NULL;
-		} else if (lines[i][0] == '\\' && lines[i][1] == '\\') {
+		} else if (line[0] == '\\' && line[1] == '\\') {
 			/* ... Or it's in the windows smb form
 			 * (\\machine\share\filename), Note drive names
 			 * (C:\ D:\ etc) are unhandled (unknown base for
@@ -446,9 +439,9 @@ totem_pl_parser_add_m3u (TotemPlParser *parser,
 		        char *tmpuri;
 
 			lines[i] = g_strdelimit (lines[i], "\\", '/');
-			tmpuri = g_strjoin (NULL, "smb:", lines[i], NULL);
+			tmpuri = g_strjoin (NULL, "smb:", line, NULL);
 
-			totem_pl_parser_add_one_uri (parser, lines[i],
+			totem_pl_parser_add_one_uri (parser, line,
 					totem_pl_parser_get_extinfo_title (extinfo));
 			extinfo = NULL;
 
@@ -459,10 +452,10 @@ totem_pl_parser_add_m3u (TotemPlParser *parser,
 			char sep;
 
 			_base_file = g_file_get_parent (file);
-			sep = (split_char[0] == '\n' ? '/' : '\\');
+			sep = (dos_mode ? '\\' : '/');
 			if (sep == '\\')
 				lines[i] = g_strdelimit (lines[i], "\\", '/');
-			uri = g_file_get_child (_base_file, lines[i]);
+			uri = g_file_get_child (_base_file, line);
 			g_object_unref (_base_file);
 			totem_pl_parser_add_one_file (parser, uri,
 						     totem_pl_parser_get_extinfo_title (extinfo));
@@ -476,17 +469,30 @@ totem_pl_parser_add_m3u (TotemPlParser *parser,
 	return retval;
 }
 
+TotemPlParserResult
+totem_pl_parser_add_m4u (TotemPlParser *parser,
+			 GFile *file,
+			 GFile *base_file,
+			 TotemPlParseData *parse_data,
+			 gpointer data)
+{
+	return totem_pl_parser_add_m3u (parser, file,
+					base_file, parse_data, data);
+}
+
 TotemPlParserResult
 totem_pl_parser_add_ra (TotemPlParser *parser,
 			GFile *file,
-			GFile *base_file, gpointer data)
+			GFile *base_file,
+			TotemPlParseData *parse_data,
+			gpointer data)
 {
 	if (data == NULL || totem_pl_parser_is_uri_list (data, strlen (data)) == NULL) {
 		totem_pl_parser_add_one_file (parser, file, NULL);
 		return TOTEM_PL_PARSER_RESULT_SUCCESS;
 	}
 
-	return totem_pl_parser_add_ram (parser, file, NULL);
+	return totem_pl_parser_add_ram (parser, file, parse_data, NULL);
 }
 
 #endif /* !TOTEM_PL_PARSER_MINI */
