@@ -25,7 +25,7 @@
 #include <string.h>
 #include <glib.h>
 #include <glib/gi18n-lib.h>
-#include <gtk/gtk.h>
+
 #include "totem-pl-parser.h"
 #include "totemplparser-marshal.h"
 #endif /* !TOTEM_PL_PARSER_MINI */
@@ -45,22 +45,24 @@
 
 #ifndef TOTEM_PL_PARSER_MINI
 gboolean
-totem_pl_parser_write_pla (TotemPlParser *parser, GtkTreeModel *model,
-			   TotemPlParserIterFunc func, 
-			   GFile *output, const char *title,
-			   gpointer user_data, GError **error)
+totem_pl_parser_save_pla (TotemPlParser    *parser,
+                          TotemPlPlaylist  *playlist,
+                          GFile            *output,
+                          const char       *title,
+                          GError          **error)
 {
+        TotemPlPlaylistIter iter;
 	GFileOutputStream *stream;
-	int num_entries_total, i;
+        gint num_entries_total, i;
 	char *buffer;
-	gboolean ret;
-
-	num_entries_total = gtk_tree_model_iter_n_children (model, NULL);
+	gboolean valid, ret;
 
 	stream = g_file_replace (output, NULL, FALSE, G_FILE_CREATE_NONE, NULL, error);
 	if (stream == NULL)
 		return FALSE;
 
+        num_entries_total = totem_pl_playlist_size (playlist);
+
 	/* write the header */
 	buffer = g_malloc0 (RECORD_SIZE);
 	*((gint32 *)buffer) = GINT32_TO_BE (num_entries_total);
@@ -78,25 +80,31 @@ totem_pl_parser_write_pla (TotemPlParser *parser, GtkTreeModel *model,
 	}
 
 	ret = TRUE;
-	for (i = 1; i <= num_entries_total; i++) {
-		GtkTreeIter iter;
-		char *euri, *title, *path, *converted, *filename;
+        valid = totem_pl_playlist_iter_first (playlist, &iter);
+        i = 0;
+
+        while (valid)
+        {
+		char *euri, *path, *converted, *filename;
 		gsize written;
-		gboolean custom_title;
 
-		if (gtk_tree_model_iter_nth_child (model, &iter, NULL, i - 1) == FALSE)
-			continue;
+                totem_pl_playlist_get (playlist, &iter,
+                                       TOTEM_PL_PARSER_FIELD_URI, &euri,
+                                       NULL);
 
-		func (model, &iter, &euri, &title, &custom_title, user_data);
-		g_free (title);
+                valid = totem_pl_playlist_iter_next (playlist, &iter);
 
-		memset (buffer, 0, RECORD_SIZE);
+                if (!euri) {
+                        continue;
+                }
 
-		/* convert to filename */
+                memset (buffer, 0, RECORD_SIZE);
 		path = g_filename_from_uri (euri, NULL, error);
+                i++;
+
 		if (path == NULL)
 		{
-			DEBUG(NULL, g_print ("Couldn't convert URI '%s' to a filename: %s\n", euri, (*error)->message));
+			DEBUG1(g_print ("Couldn't convert URI '%s' to a filename: %s\n", euri, (*error)->message));
 			g_free (euri);
 			ret = FALSE;
 			break;
@@ -112,7 +120,9 @@ totem_pl_parser_write_pla (TotemPlParser *parser, GtkTreeModel *model,
 		{
 			/* should never happen, but this would be the right value */
 			buffer[1] = 0x01;
-		} else {
+		}
+                else
+                {
 			/* we want the char after the slash, and it's one-based */
 			guint name_offset = (filename - path) + 2;
 
@@ -127,7 +137,7 @@ totem_pl_parser_write_pla (TotemPlParser *parser, GtkTreeModel *model,
 		converted = g_convert (path, -1, "UTF-16BE", "UTF-8", NULL, &written, error);
 		if (converted == NULL)
 		{
-			DEBUG(NULL, g_print ("Couldn't convert filename '%s' to UTF-16BE\n", path));
+			DEBUG1(g_print ("Couldn't convert filename '%s' to UTF-16BE\n", path));
 			g_free (path);
 			ret = FALSE;
 			break;
@@ -142,7 +152,7 @@ totem_pl_parser_write_pla (TotemPlParser *parser, GtkTreeModel *model,
 
 		if (totem_pl_parser_write_buffer (G_OUTPUT_STREAM (stream), buffer, RECORD_SIZE, error) == FALSE)
 		{
-			DEBUG(NULL, g_print ("Couldn't write entry %d to the file\n", i));
+			DEBUG1(g_print ("Couldn't write entry %d to the file\n", i));
 			ret = FALSE;
 			break;
 		}
@@ -210,7 +220,7 @@ totem_pl_parser_add_pla (TotemPlParser *parser,
 				  NULL, NULL, &error);
 		if (path == NULL)
 		{
-			DEBUG(NULL, g_print ("error converting entry %d to UTF-8: %s\n", entry, error->message));
+			DEBUG1(g_print ("error converting entry %d to UTF-8: %s\n", entry, error->message));
 			g_error_free (error);
 			retval = TOTEM_PL_PARSER_RESULT_ERROR;
 			break;
@@ -223,7 +233,7 @@ totem_pl_parser_add_pla (TotemPlParser *parser,
 		uri = g_filename_to_uri (path, NULL, NULL);
 		if (uri == NULL)
 		{
-			DEBUG(file, g_print ("error converting path %s to URI: %s\n", path, error->message));
+			DEBUG1(g_print ("error converting path %s to URI: %s\n", path, error->message));
 			g_error_free (error);
 			retval = TOTEM_PL_PARSER_RESULT_ERROR;
 			break;
