@@ -41,15 +41,19 @@
  * thread to a stored GThread pointer known to be from the main thread
  * (TotemPlParser->priv->main_thread).
  *
+ * When using idle functions for async signal emission, we specify the same priority as
+ * GSimpleAsyncResult uses for its completion function (G_PRIORITY_DEFAULT). If the
+ * completion function has higher priority, the main loop will call it first.
+ *
  * @p: a #TotemPlParser
  * @c: callback (as if for g_idle_add())
  * @d: callback data
  */
-#define CALL_ASYNC(p, c, d) {				\
-	if (g_thread_self () == p->priv->main_thread)	\
-		c (d);					\
-	else						\
-		g_idle_add ((GSourceFunc) c, d);	\
+#define CALL_ASYNC(p, c, d) {							\
+	if (g_thread_self () == p->priv->main_thread)				\
+		c (d);								\
+	else									\
+		g_idle_add_full (G_PRIORITY_DEFAULT, (GSourceFunc) c, d, NULL);	\
 }
 
 #ifndef TOTEM_PL_PARSER_MINI
@@ -80,7 +84,6 @@
 #define TOTEM_PL_PARSER_FIELD_FILE		"gfile-object"
 #define TOTEM_PL_PARSER_FIELD_BASE_FILE		"gfile-object-base"
 
-#ifndef TOTEM_PL_PARSER_MINI
 #define DEBUG(file, x) {					\
 	if (totem_pl_parser_is_debugging_enabled (parser)) {	\
 		if (file != NULL) {				\
@@ -95,9 +98,11 @@
 		}						\
 	}							\
 }
-#else
-#define DEBUG(x) { if (totem_pl_parser_is_debugging_enabled (parser)) x; }
-#endif
+#define DEBUG1(x) {						\
+	if (totem_pl_parser_is_debugging_enabled (parser)) {	\
+		x;						\
+	}							\
+}
 
 typedef struct {
 	guint recurse_level;
